(* This is the source calculus of the sample client. It is a core ML. *)

(* Nominal representation of term variables, type variables and binders. *)

type tevar = string
type tyvar = string

type typ =
  | TyVar of tyvar
  | TyArrow of typ * typ
  | TyProduct of typ list
  | TyConstr of Datatype.tyconstr_id * typ list

type datatype_env = (tyvar, typ option) Datatype.Env.t

type type_annotation = tyvar list * typ
  (* some <flexible vars> . <ty> *)

type term =
  | Var of tevar
  | Hole of term list
  | Abs of tevar * term
  | App of term * term
  | Let of tevar * term * term
  | Annot of term * type_annotation
  | Tuple of term list
  | LetProd of tevar list * term * term
  | Variant of Datatype.label_id * term option
  | Match of term * branch list

and branch = pattern * term

and pattern =
  | PVar of tevar
  | PWildcard
  | PAnnot of pattern * type_annotation
  | PVariant of Datatype.label_id * pattern option
  | PTuple of pattern list
